
----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I


-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = "Clock" -- L["Clock"]
local ldbName = name
local tt, tooltip
local GetGameTime = GetGameTime
local GetGameTime2
local countries = {}
local played = {}
local clock_diff = nil

-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
I[name] = {iconfile="Interface\\Addons\\"..addon.."\\media\\clock"}


---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name] = {
	desc = L["Broker to show realm or local time"],
	events = {
		"TIME_PLAYED_MSG"
	},
	updateinterval = 1,
	config_defaults = {
		format24 = true,
		timeLocal = true,
		showSeconds = false
	},
	config_allowed = {
	},
	config = {
		height = 100,
		elements = {
			{
				type = "check",
				name = "format24",
				label = L["24 hours mode"], -- TIMEMANAGER_24HOURMODE
				desc = L["Switch between time format 24 hours and 12 hours with AM/PM"],
			},
			{
				type = "check",
				name = "timeLocal",
				label = L["Local or server time"],
				desc = L["Switch between local and server time in broker button"],
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x = 200,
					y = 0
				}
			},
			{
				type = "check",
				name = "showSeconds",
				label = L["Show seconds"],
				desc = L["Display the time with seconds in broker button and tooltip"],
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x = 200,
					y = 0
				},
				disabled = true
			},
			--{ type="next_row" },
			{
				type = "dropdown",
				name = "inBroker",
				label = L["Time in broker"],
				desc = L["Choose which time you like to see in broker."],
				default = "s",
				values = {
					["s"] = L["Server time"],
					["l"] = L["Local time"],
					["sl"] = L["Server / Local time"],
					["ls"] = L["Local / Server time"]
				},
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x = -10,
					y = -22
				},
				disabled = true
			},
		}
	}
}


--------------------------
-- some local functions --
--------------------------
local function _seconds(n,t)
	local x,y,z = 0,0,{day=86400,hour=3600,minute=60}
	if n>=t then
		x = n / z[t]
		y = floor(x)
		n = n - (y * z[t])
	end
	return y, n
end

local function getTime()
	local s,l,u,p = {GetGameTime()},{date("!%H"),{date("%H"),date("%M"),date("%S")},date("!%M"),date("!%S")},{t={},l={}}
	tinsert(s,date("%S"))
	if played.stamp~=nil then
		local t, l, c = played.total + (time()-played.stamp), played.level + (time()-played.stamp),0 -- rise up played time without use of RequestTimePlayed()

		p.t.d,t = _seconds(t,"day")		t=t-c
		p.t.h,t = _seconds(t,"hour")	t=t-c
		p.t.m,t = _seconds(t,"minute")	t=t-c
		p.t.s = t-cut

		p.l.d,l = _seconds(l,"day")		l=l-c
		p.l.h,l = _seconds(l,"hour")	l=l-c
		p.l.m,l = _seconds(l,"minute")	l=l-c
		p.l.s = l-cut

	end
	return s,l,u,p
end

local function countryTime(utc)
	
end

local function getTimeStrings()
	local h24 = Broker_EverythingDB[name].format24
	local s,l,seconds = nil,nil,tonumber(date("%S"))
	local h,m,S = ns.GetGameTime()
	local a = nil
	if (not h24) then
		a = "AM"
		if h >= 12 then h,a = h-12,"PM" end
		if h == 0 then h = 12 end
	end

	if Broker_EverythingDB[name].showSeconds then
		s=(h24 and "%02d:%02d:%02s" or "%02d:%02d:%02s %s"):format(h,m,S,a)
		l=date(h24 and "%H:%M:%S" or "%I:%M:%S %p")
	else
		s=(h24 and "%02d:%02d" or "%02d:%02d %s"):format(h,m,a)
		l=date(h24 and "%H:%M" or "%I:%M %p")
	end
	return s,l
end

------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(obj)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
end

ns.modules[name].onevent = function(self,event,...)
	if event == "TIME_PLAYED_MSG" then
		played.total, played.level = ...
		played.stamp = time()
	end
end

--[[ ns.modules[name].optionspanel = function(panel) end ]]

--[[ ns.modules[name].onmousewheel = function(self,direction) end ]]

ns.modules[name].onupdate = function(self)
	if not self then self = {} end
	self.obj = self.obj or ns.LDB:GetDataObjectByName(ldbName)

	local _server,_local = getTimeStrings()

	self.obj.text = Broker_EverythingDB[name].timeLocal and _local or _server
end

ns.modules[name].ontooltip = function(tt)
	tooltip=tt
	ns.tooltipScaling(tt)
	local _server,_local = getTimeStrings()
	tt:ClearLines()

	tt:AddLine(L[name])
	tt:AddLine(" ")

	tt:AddDoubleLine(C("white",L["Local Time"]), C("white",_local))
	tt:AddDoubleLine(C("white",L["Server Time"]), C("white",_server))

	if Broker_EverythingDB.showHints then
		tt:AddLine(" ")
		tt:AddLine(C("copper",L["Left-click"]).." || "..C("green",L["Open time manager"]))
		tt:AddLine(C("copper",L["Right-click"]).." || "..C("green",L["Local or server time"]))
		tt:AddLine(C("copper",L["Shift+Right-click"]).." || "..C("green",L["Open calendar"]))
		tt:AddLine(C("copper",L["Shift+Left-click"]).." || "..C("green",L["12 / 24 hours mode"]))
	end
end


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
--[[ ns.modules[name].onenter = function(self) end ]]

--[[ ns.modules[name].onleave = function(self) end ]]

ns.modules[name].onclick = function(self,button)
	local shift = IsShiftKeyDown()
	if button == "RightButton" and not shift then
		if Broker_EverythingDB[name].timeLocal == true then
			Broker_EverythingDB[name].timeLocal = false
		else
			Broker_EverythingDB[name].timeLocal = true
		end
		ns.modules[name].onupdate(self)
	elseif button == "RightButton" and shift then
		securecall("ToggleCalendar")
	elseif button == "LeftButton" and shift then 
		if Broker_EverythingDB[name].format24 == true then
			Broker_EverythingDB[name].format24 = false
		else
			Broker_EverythingDB[name].format24 = true
		end
		ns.modules[name].onupdate(self)
	else
		securecall("ToggleTimeManager")
	end
end

--[[ ns.modules[name].ondblclick = function(self,button) end ]]




--[[

Time
-------------------
Server
Local

Playtime
-------------------
playtime total
playtime level

Other Countries
-------------------
<5 chooseable>




]]

