
----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I


-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = "Currency" -- L["Currency"]
local ldbName = name
local tt = nil
local tt2 = nil
local tt3 = nil
local currencies = {}
local currencies_num = 0
local currenciesName2Id = {}
local menu = nil
local tt2positions = {
	["BOTTOM"] = {edgeSelf = "TOP",    edgeParent = "BOTTOM", x =  0, y = -2},
	["LEFT"]   = {edgeSelf = "RIGHT",  edgeParent = "LEFT",   x = -2, y =  0},
	["RIGHT"]  = {edgeSelf = "LEFT",   edgeParent = "RIGHT",  x =  2, y =  0},
	["TOP"]    = {edgeSelf = "BOTTOM", edgeParent = "TOP",    x =  0, y =  2}
}


-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
--I[name..'_Neutral']  = {iconfile="Interface\\Addons\\"..addon.."\\media\\icon-Neutral"}
I[name..'_Neutral']  = {iconfile="Interface\\minimap\\tracking\\BattleMaster", coords={0.1,0.9,0.1,0.9}}
I[name..'_Horde']    = {iconfile="Interface\\PVPFrame\\PVP-Currency-Horde", coords={0.1,0.9,0.1,0.9}}
I[name..'_Alliance'] = {iconfile="Interface\\PVPFrame\\PVP-Currency-Alliance", coords={0.1,0.9,0.1,0.9}}

---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name] = {
	desc = L["Broker to show your different currencies."],
	icon_suffix = "_Neutral",
	events = {
		"PLAYER_ENTERING_WORLD",
		"KNOWN_CURRENCY_TYPES_UPDATE",
		"CURRENCY_DISPLAY_UPDATE"
	},
	updateinterval = nil, -- 10
	config_defaults = {
		shortTT = false,
		subTTposition = "AUTO",
		currenciesInTitle = {["1"]=false,["2"]=false,["3"]=false,["4"]=false}
	},
	config_allowed = {
		subTTposition = {["AUTO"]=true,["TOP"]=true,["LEFT"]=true,["RIGHT"]=true,["BOTTOM"]=true}
	},
	config = {
		height = 52,
		elements = {
			{
				type = "check",
				name = "shortTT",
				label = L["short Tooltip"],
				desc = L["display the content of the tooltip shorter"]
			},
			{
				type	= "dropdown",
				name	= "subTTposition",
				label	= L["Second tooltip"],
				desc	= L["Where does the second tooltip for a single currency are displayed from the first tooltip"],
				values	= {
					["AUTO"]    = L["Auto"],
					["TOP"]     = L["Over"],
					["LEFT"]    = L["Left"],
					["RIGHT"]   = L["Right"],
					["BOTTOM"]  = L["Under"]
				},
				default = "BOTTOM",
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x = 200,
					y = 0
				}
			}
		}
	}
}


--------------------------
-- some local functions --
--------------------------
local function collectData() -- collect currency data
	local byName = {}
	currencies_num = 0
	for i=1, GetCurrencyListSize() do
		local itemname, isHeader, isExpanded, isUnused, isWatched, count, icon = GetCurrencyListInfo(i)

		if not isUnused then
			currencies[i] = {
				["name"] = itemname,
				["isHeader"] = isHeader,
				["isUnused"] = isUnused,
				["count"] = count,
				["maxCount"] = nil,
				["maxWeekly"] = nil,
				["icon"] = icon
			}
			if itemname then
				currenciesName2Id[itemname] = i
			end
			if not isHeader then
				currencies_num=currencies_num+1
			end
		end
	end
end

local function updateTitle()
	local title = {}
	for i, v in pairs(Broker_EverythingDB[name].currenciesInTitle) do
		if v and currenciesName2Id[v]~=nil and currencies[currenciesName2Id[v]]~=nil then
			local d = currencies[currenciesName2Id[v]]
			if d.icon then
				table.insert(title, d.count .. "|T" .. d.icon .. ":0|t")
			end
		--else
		--	Broker_EverythingDB[name].currenciesInTitle[i] = false
		end
	end
	local obj = ns.LDB:GetDataObjectByName(ldbName)
	if #title==0 then
		obj.text = L[name]
	else
		obj.text = table.concat(title," ")
	end
end

local function setInTitle(titlePlace, currencyName)
	if Broker_EverythingDB[name].currenciesInTitle[titlePlace] ~= currencyName then
		for i, v in pairs(Broker_EverythingDB[name].currenciesInTitle) do
			if titlePlace~=i and Broker_EverythingDB[name].currenciesInTitle[i]==currencyName then return end
		end
		Broker_EverythingDB[name].currenciesInTitle[titlePlace] = currencyName
	else
		Broker_EverythingDB[name].currenciesInTitle[titlePlace] = false
	end
	updateTitle()
	ns.hideTooltip(tt) -- shared.lua
end

local function makeMenu(parent)
	if tt~=nil and tt:IsShown() then ns.hideTooltip(tt) end
	local menuTable = {{text = L["Currencies in Title"], isTitle = true }}
	local i = 2
	local inTitle = setmetatable({},{__index=function() return false end})
	menu = CreateFrame("Frame",name.."_Dropdown", parent or UIParent, "UIDropDownMenuTemplate")
	
	collectData()

	for place=1, 4 do
		if Broker_EverythingDB[name].currenciesInTitle[place] then
			inTitle[Broker_EverythingDB[name].currenciesInTitle[place]] = true
		end
	end

	for place=1, (currencies_num>=4) and 4 or currencies_num do
		menuTable[i],i = {text = C("white",L["Place"].." "..place), isTitle = true },i+1
		local doing = L["Add a currency"]
		if Broker_EverythingDB[name].currenciesInTitle[place] then
			local id = currenciesName2Id[Broker_EverythingDB[name].currenciesInTitle[place]]
			local d = currencies[id]
			if d~=nil then
				menuTable[i],i = { text = L["Remove"].." "..C("ltblue",d.name), func = function() setInTitle(place,d.name); menu:Hide(); menu = nil end },i+1
				doing = L["Replace the curreny"]
			end
		end
		menuTable[i] = { text = doing, hasArrow = true, menuList = {} }
		local I = 1
		for k,v in pairs(currencies) do
			if v.name and not inTitle[v.name] then
				if v.isHeader then
					menuTable[i].menuList[I],I = {text = C("ltblue",v.name), isTitle = true},I+1
				else
					menuTable[i].menuList[I],I = {text = C("ltyellow",v.name), func = function() setInTitle(place,v.name); menu:Hide(); menu = nil end},I+1
				end
			end
		end
		i=i+1
	end

	EasyMenu(menuTable,menu,parent or "cursor",0,0,"MENU")
end



------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(self)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
	if self then
		updateTitle()
	end
end

ns.modules[name].onevent = function(self,event,msg)
	local obj = ns.LDB:GetDataObjectByName(ldbName)
	if UnitFactionGroup("player") ~= "Neutral" then
		local i = I(name.."_"..UnitFactionGroup("player"))
		obj.iconCoords = i.coords or {0,1,0,1}
		obj.icon = i.iconfile
	end
	collectData()
	updateTitle()
end

--[[ ns.modules[name].onupdate = function(self) end ]]

--[[ ns.modules[name].optionspanel = function(panel) end ]]

--[[ ns.modules[name].onmousewheel = function(self,direction) end ]]

ns.modules[name].ontooltip = function(tt)

	tt:Clear()
	tt:AddHeader(C("dkyellow",L[name]))
	if Broker_EverythingDB[name].shortTT == true then
		tt:AddSeparator()
	end

	for i,v in ipairs(currencies) do
		if not v.isUnused then
			if v.isHeader then
				if Broker_EverythingDB[name].shortTT == false then
					tt:AddSeparator(3,0,0,0,0)
					tt:AddLine(C("ltblue",v.name))
					tt:AddSeparator()
				end
			else
				local line, column = tt:AddLine(C("ltyellow",v.name),v.count.."  |T"..v.icon..":0|t")
				local lineObj = tt.lines[line]
				lineObj.currencyIndex = i

				tt:SetLineScript(line, "OnEnter",function(self)
					local pos = {}
					if not tt2 then
						tt2 = GameTooltip
					end
					tt2:SetOwner(tt,"ANCHOR_NONE")

					if Broker_EverythingDB[name].subTTposition == "AUTO" then
						local tL,tR,tT,tB = ns.getBorderPositions(tt)
						local uW = UIParent:GetWidth()
						if tB<200 then
							pos = tt2positions["TOP"]
						elseif tL<200 then
							pos = tt2positions["RIGHT"]
						elseif tR<200 then
							pos = tt2positions["LEFT"]
						else
							pos = tt2positions["BOTTOM"]
						end
					else
						pos = tt2positions[Broker_EverythingDB[name].subTTposition]
					end

					tt2:SetPoint(pos.edgeSelf,tt,pos.edgeParent, pos.x , pos.y)
					-- changes for user choosen direction
					tt2:ClearLines()
					tt2:SetCurrencyToken(self.currencyIndex) -- tokenId / the same index number if needed by GetCurrencyListInfo
					tt2:Show()
				end)

				tt:SetLineScript(line, "OnLeave", function(self)
					tt2:Hide()
				end)
			end
		end
	end

	if Broker_EverythingDB.showHints then
		tt:AddSeparator(3,0,0,0,0)
		local l,c = tt:AddLine()
		tt:SetCell(l,1,
			C("copper",L["Left-click"]).." || "..C("green",L["Open currency pane"])
			.."|n"..
			C("copper",L["Right-click"]).." || "..C("green",L["Currency in title - menu"]),
			nil,
			nil,
			2)
	end
end


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
ns.modules[name].onenter = function(self)
	tt = ns.LQT:Acquire(name.."TT", 2, "LEFT", "RIGHT")
	tt:Clear()
	ns.modules[name].ontooltip(tt)
	ns.createTooltip(self,tt)
end

ns.modules[name].onleave = function(self)
	if tt then
		if MouseIsOver(tt) then
			tt:SetScript('OnLeave', function(self) ns.hideTooltip(self,name) end)
		else
			ns.hideTooltip(tt,name)
		end
	end
end

ns.modules[name].onclick = function(self,button)
	if button == "LeftButton" then
		securecall("ToggleCharacter","TokenFrame")
	else
		if menu==nil then
			makeMenu(self)
		else
			menu:Hide()
			menu = nil
		end
	end
end

--[[ ns.modules[name].ondblclick = function(self,button) end ]]




--[[
IDEAS:
* get max count and weekly max count of a currency for displaying caped counts in red.
* Ctrl-clicks to set custom caps.
]]

