
----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I
graphicsSetsDB = {}

-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = "FPS" -- L["FPS"]
local ldbName = name
local tt,tt2 = nil,nil
local GetFramerate = GetFramerate
local _, playerClass = UnitClass("player")
local _minmax = {[1] = nil,[2] = nil}
local graph_maxValues,graph_maxHeight = 50,50
local fps = 0
local minmax_delay = 3
local need_reload = false
local need_reload_cvars = {}
local tt2_content = {
	{type="head",name="DISPLAY_HEADER"},
	{type="sep"},
	{type="line",name="DISPLAY_MODE",			cvar="", cvarType=""},
	{type="line",name="PRIMARY_MONITOR",		cvar="", cvarType=""},
	{type="line",name="MULTISAMPLE",			cvar="", cvarType=""},
	{type="line",name="VERTICAL_SYNC",			cvar="", cvarType="boolean"},

	{type="sep", params={3,0,0,0,0}},
	{type="head",name="TEXTURES_SUBHEADER"},
	{type="sep"},
	{type="line",name="TEXTURE_DETAIL",			cvar="", cvarType=""},
	{type="line",name="ANISOTROPIC",			cvar="", cvarType=""},
	{type="line",name="PROJECTED_TEXTURES",		cvar="", cvarType="boolean"},

	{type="sep", params={3,0,0,0,0}},
	{type="head",name="EFFECTS_SUBHEADER"},
	{type="sep"},
	{type="line",name="SHADOW_QUALITY",			cvar="", cvarType=""},
	{type="line",name="LIQUID_DETAIL",			cvar="", cvarType=""},
	{type="line",name="SUNSHAFTS",				cvar="", cvarType=""},
	{type="line",name="PARTICLE_DENSITY",		cvar="", cvarType=""},
	{type="line",name="SSAO_LABEL",				cvar="", cvarType=""},

	{type="sep", params={3,0,0,0,0}},
	{type="head",name="ENVIRONMENT_SUBHEADER"},
	{type="sep"},
	{type="line",name="FARCLIP",				cvar="", cvarType=""},
	{type="line",name="ENVIRONMENT_DETAIL",		cvar="", cvarType=""},
	{type="line",name="GROUND_CLUTTER",			cvar="", cvarType=""},

	{type="sep", params={3,0,0,0,0}},
	{type="head",name="EFFECTS_LABEL"},
	{type="sep"},
	{type="line",name="TRIPLE_BUFFER",			cvar="", cvarType=""},
	{type="line",name="FIX_LAG",				cvar="", cvarType=""},
	{type="line",name="HARDWARE_CURSOR",		cvar="", cvarType=""},
	{type="line",name="GXAPI",					cvar="", cvarType=""},
}

-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
I[name..'_yellow'] = {iconfile="Interface\\Addons\\"..addon.."\\media\\fps_yellow"}
I[name..'_red']    = {iconfile="Interface\\Addons\\"..addon.."\\media\\fps_red"}
I[name..'_blue']   = {iconfile="Interface\\Addons\\"..addon.."\\media\\fps_blue"}
I[name..'_green']  = {iconfile="Interface\\Addons\\"..addon.."\\media\\fps_green"}


---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name] = {
	desc = L["Broker to show your frames per second."],
	icon_suffix = "_blue",
	events = {},
	updateinterval = 1,
	config_defaults = nil,
	config_allowed = nil,
	config = nil -- {}
}


--------------------------
-- some local functions --
--------------------------
local function minmax(f)
	if minmax_delay~=0 then minmax_delay = minmax_delay - 1 return end
	if (not _minmax[1]) or _minmax[1] > f then _minmax[1]=f end
	if (not _minmax[2]) or _minmax[2] < f then _minmax[2]=f end
end

local function fps_color(f)
	if not f then return {"","","?"} end
	local c = (f<20 and {"_red","red"}) or (f<30 and {"_yellow","dkyellow"}) or (f<100 and {"_green","green"}) or {"_blue","ltblue"}
	table.insert(c,C(c[2],f)..C("suffix","fps"))
	return c
end

local function fpsTooltip(tt)
	local l, c, cell
	tt:Clear()
	tt:AddHeader(C("dkyellow",L[name]))
	tt:AddSeparator()
	tt:AddLine(L["Current"]..":",fps_color(fps)[3])
	tt:AddSeparator(3,0,0,0,0)
	tt:AddLine(L["Min."]..":",fps_color(_minmax[1])[3])
	tt:AddLine(L["Max."]..":",fps_color(_minmax[2])[3])

	--[[
	tt:AddSeparator(3,0,0,0,0)
	l,c = tt:AddLine()
	tt:SetCell(l,1,"",nil,nil,2)
	local cell = tt.lines[l].cells[1]

	local f = ns.tooltipGraph(name,graph_maxValues,graph_maxHeight,true)
	f:SetParent(cell)
	f:SetPoint("TOPLEFT", cell,"TOPLEFT", 0, 0)
	cell:SetWidth(f:GetWidth())
	cell:SetHeight(f:GetHeight())
	]]

	--if Broker_EverythingDB.showHints then
	if false then
		tt:AddLine(" ")
		tt:AddLine(C("copper",L["Click"]).." ||",C("green",L["Open graphics set manager"]))
		tt:AddLine(C("copper",L["Right-Click"]).." ||",C("green",L["Open graphics menu"]))
	end

end

local function graphicsSetManager(_self)
	if tt~=nil and tt:IsShown() then ns.hideTooltip(tt) end
	tt2 = ns.LQT:Acquire(name.."TT2", 1, "LEFT")
	ns.createTooltip(_self,tt2)
	tt2:SetScript('OnLeave', ns.hideTooltip)
	tt2:Clear()

	tt2:AddHeader(C("ltblue","Graphics set manager"))
	tt2:AddSeparator()
	tt2:AddLine(L["No set found"])

	if Broker_EverythingDB.showHints then
		tt2:AddLine(" ")
		line, column = tt:AddLine()
		tt2:SetCell(line, 1,
			C("ltblue",L["Click"]).." || "..C("green",L["Use a set"])
			.."|n"..
			C("ltblue",L["Shift+Click"]).." || "..C("green",L["Update/save a set"])
			.."|n"..
			C("ltblue",L["Ctrl+Click"]).." || "..C("green",L["Delete a set"])
			, nil, nil, 1)
		
	end
end

local function graphicsMenu(_self)
	local l,c
	if tt~=nil and tt:IsShown() then ns.hideTooltip(tt) end
	tt2 = ns.LQT:Acquire(name.."TT2", 2, "LEFT", "RIGHT")
	ns.createTooltip(_self,tt2)
	tt2:SetScript('OnLeave', ns.hideTooltip)
	tt2:Clear()

	for i,v in ipairs(tt2_content) do
		if v.type=="sep" then
			if type(v.params)=="table" then
				tt2:AddSeparator(unpack(v.params))
			else
				tt2:AddSeparator()
			end
		elseif v.type=="head" then
			tt2:AddLine(C("ltblue",_G[v.name]))
		elseif v.type=="line" then
			l,c = tt2:AddLine(C("ltyellow",_G[v.name]),"?")
		end
	end
end

local function getSettings()
	
end

------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(obj)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
end

--[[ ns.modules[name].onevent = function(self,event,msg) end ]]

ns.modules[name].onupdate = function(self)
	fps = floor(GetFramerate())
	local c = fps_color(fps)
	local d = self.obj or ns.LDB:GetDataObjectByName(ldbName)

	minmax(fps)
	--ns.tooltipGraphAddValue(name,fps,graph_maxValues)

	local icon = I(name..c[1])
	d.iconCoords = icon.coords or {0,1,0,1}
	d.icon = icon.iconfile
	d.text = c[3]

	--if tt then
	if tt~=nil and tt.key~=nil and tt.key==name.."TT" and tt:IsShown() then
		fpsTooltip(tt)
	end
end

--[[ ns.modules[name].optionspanel = function(panel) end ]]

--[[ ns.modules[name].onmousewheel = function(self,direction) end ]]

--[[ ns.modules[name].ontt = function(tt) end ]]

-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
ns.modules[name].onenter = function(self)
	tt = ns.LQT:Acquire(name.."TT", 2, "LEFT", "RIGHT")
	fpsTooltip(tt)
	ns.createTooltip(self,tt)
end

ns.modules[name].onleave = function(self)
	if tt~=nil then
		if MouseIsOver(tt) then
			tt:SetScript('OnLeave', function(self)
				ns.hideTooltip(self,name)
				tt = nil
			end)
		else
			ns.hideTooltip(tt,name)
			tt = nil
		end
	end
end

ns.modules[name].onclick = function(self,button)
	if button == "LeftButton" then
		--graphicsSetManager(self)
	elseif button == "RightButton" then
		--graphicsMenu(self)
	end
end

--[[ ns.modules[name].ondblclick = function(self,button) end ]]

