
-- saved variables
goldDB = {}

----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I


-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = "Gold" -- L["Gold"]
local ldbName = name
local tt = nil
local login_money = nil
local next_try = false
local current_money = 0
local goldInit = false
local goldLoaded = false
local faction = UnitFactionGroup("Player")


-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
I[name] = {iconfile="Interface\\Minimap\\TRACKING\\Auctioneer",coords={0.05,0.95,0.05,0.95}}


---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name] = {
	desc = L["Broker to show gold information. Shows gold amounts for characters on the same ns.realm and faction and the amount made or lost for the session."],
	events = {
		"PLAYER_LOGIN",
		"PLAYER_MONEY",
		"PLAYER_TRADE_MONEY",
		"TRADE_MONEY_CHANGED",
		"PLAYER_ENTERING_WORLD",
		"NEUTRAL_FACTION_SELECT_RESULT"
	},
	updateinterval = nil, -- 10
	config_defaults = {
		goldColor = false
	},
	config_allowed = {
	},
	config = {
		height = 52,
		elements = {
			{
				type = "check",
				name = "goldColor",
				label = L["Gold coloring"],
				desc = L["Use colors instead of icons for gold, silver and copper"],
				event = true
			}
		}
	}
}


--------------------------
-- some local functions --
--------------------------


------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(obj)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
	if not goldDB[faction] then
		goldDB[faction] = { [ns.realm]={ [ns.player.name] = 0 } }
	elseif not goldDB[faction][ns.realm] then
		goldDB[faction][ns.realm] = { [ns.player.name] = 0 }
	end
end

ns.modules[name].onevent = function(self,event,msg)
	current_money = GetMoney()
	goldDB[faction][ns.realm][ns.player.name] = current_money

	if event=="PLAYER_LOGIN" or (next_try and login_money==nil) then
		login_money = current_money
		next_try = (next_try==false and login_money==nil)
	end

	if event == "NEUTRAL_FACTION_SELECT_RESULT" then
		faction = UnitFactionGroup("Player")
		ns.modules[name].init(self)
		goldDB[faction][ns.realm][ns.player.name] = current_money
		goldDB["Neutral"][ns.realm][ns.player.name] = nil
	end

	(self.obj or ns.LDB:GetDataObjectByName(ldbName)).text = ns.GetCoinColorOrTextureString(name,current_money)
end

--[[ ns.modules[name].onupdate = function(self) end ]]

--[[ ns.modules[name].optionspanel = function(panel) end ]]

--[[ ns.modules[name].onmousewheel = function(self,direction) end ]]

ns.modules[name].ontooltip = function(tt)
	local totalGold = 0
	local diff_money

	current_money = GetMoney()
	goldDB[faction][ns.realm][ns.player.name] = current_money

	tt:Clear()

	tt:AddHeader(C("dkyellow",L["Gold information"]))
	tt:AddSeparator()

	for k,v in ns.pairsByKeys(goldDB[faction][ns.realm]) do
		local line, column = tt:AddLine(ns.scm(k), ns.GetCoinColorOrTextureString(name,v))
		tt:SetLineScript(line, "OnMouseUp", function(self,x,button)
			if button == "RightButton" then
				goldDB[faction][ns.realm][k] = nil
				tt:Clear()
				ns.modules[name].ontooltip(tt)
			end 
		end)
		tt:SetLineScript(line, "OnEnter", function(self) tt:SetLineColor(line, 1,192/255, 90/255, 0.3) end )
		tt:SetLineScript(line, "OnLeave", function(self) tt:SetLineColor(line, 0,0,0,0) mButton=nil end)
		totalGold = totalGold + v
		line, column = nil, nil
	end

	tt:AddSeparator()
	tt:AddLine(L["Total Gold"], ns.GetCoinColorOrTextureString(name,totalGold))
	tt:AddSeparator(3,0,0,0,0)

	if login_money == nil then
		tt:AddLine(L["Session profit"], C("orange","Error"))
	elseif current_money == login_money then
		tt:AddLine(L["Session profit"], ns.GetCoinColorOrTextureString(name,0))
	elseif current_money > login_money then
		tt:AddLine(C("ltgreen",L["Session profit"]), "+ " .. ns.GetCoinColorOrTextureString(name,current_money - login_money))
	else
		tt:AddLine(C("ltred",L["Session loss"]), "- " .. ns.GetCoinColorOrTextureString(name,login_money - current_money))
	end

	if Broker_EverythingDB.showHints then
		tt:AddSeparator(3,0,0,0,0)
		line, column = tt:AddLine()
		tt:SetCell(line, 1, C("ltblue",L["Right-click"]).." || "..C("green",L["Remove entry"]), nil, nil, 2)
		line, column = tt:AddLine()
		tt:SetCell(line, 1, C("copper",L["Click"]).." || "..C("green",L["Open currency pane"]), nil, nil, 2)
	end
end


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
ns.modules[name].onenter = function(self)
	tt = ns.LQT:Acquire(name.."TT", 2, "LEFT", "RIGHT")
	ns.modules[name].ontooltip(tt)
	ns.createTooltip(self,tt)
end

ns.modules[name].onleave = function(self)
	if tt then 
		if MouseIsOver(tt) then
			tt:SetScript('OnLeave', function(self) ns.hideTooltip(self,name) end)
		else
			ns.hideTooltip(tt,name)
		end
	end
end

ns.modules[name].onclick = function(self,button)
	securecall("ToggleCharacter","TokenFrame")
end

--[[ ns.modules[name].ondblclick = function(self,button) end ]]
