
----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I
mailDB = {}

-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = "Mail" -- L["Mail"]
local ldbName = name
local tooltip = nil
local tt = nil
local icons = {}
do for i=1, 22 do local _ = ("inv_letter_%02d"):format(i) icons[_] = "|Tinterface\\icons\\".._..":16:16:0:0|t" end end

-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
I[name] = {iconfile="interface\\icons\\inv_letter_15",coords={0.05,0.95,0.05,0.95}}
--I[name..'_new'] = {iconfile="interface\\icons\\inv_letter_16",coords={0.05,0.95,0.05,0.95}}
I[name..'_new'] = {iconfile="interface\\icons\\inv_letter_18",coords={0.05,0.95,0.05,0.95}}


---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name] = {
	desc = L["Broker to alert you if you have mail."],
	events = {
		"UPDATE_PENDING_MAIL",
		"MAIL_INBOX_UPDATE",
		"MAIL_CLOSED",
		"PLAYER_LOGIN",
		"PLAYER_ENTERING_WORLD",
		"MAIL_SHOW"
	},
	updateinterval = nil, -- 10
	config_defaults = {
		playsound = false,
		showDaysLeft = true,
		hideMinimapMail = true
	},
	config_allowed = {
	},
	config = {
		height = 52, --92,
		elements = {
			{
				type = "check",
				name = "playsound",
				label = L["Play sound on new mail"],
				desc = L["Enable to play a sound on receiving a new mail message. Default is off"],
			},
			{
				type = "check",
				name = "showDaysLeft",
				label = L["Mail return to sender"],
				desc = L["Display a list of all chars on all realms with mail count and lowest days before return to sender."],
				disabled = true,
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x=200,
					y=0
				}
			},
			{
				type = "check",
				name = "hideMinimapMail",
				label = L["Hide minimap mail icon"],
				desc = L[""],
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x=200,
					y=0
				}
			},
			--[[
			{ type="next_row" },
			{
				type = "dropdown",
				name = "defaultIcon",
				label = L["default icon"],
				desc = L[""],
				default = "inv_letter_15",
				values = icons,
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x=0,
					y=-10
				}
			},
			{
				type = "onchange",
				name = "defaultIcon",
				subtype = "texture",
				width = 24,
				height = 24,
				func = function(self)
					self:SetTexture(Broker_EverythingDB[name].defaultIcon)
				end
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x=0,
					y=0
				}
			},
			{
				type = "dropdown",
				name = "pendingIcon",
				label = L["pending icon"],
				desc = L[""],
				default = "inv_letter_16",
				values = icons,
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x=30,
					y=0
				}
			},
			{
				type = "onchange",
				name = "defaultIcon",
				subtype = "texture",
				width = 24,
				height = 24,
				func = function(self)
					self:SetTexture(Broker_EverythingDB[name].defaultIcon)
				end
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x=0,
					y=0
				}
			},
			need an other way to choose the fav. icon.
			icons in normal dropdown menus are too small.
			]]
		},
	}
}


--------------------------
-- some local functions --
--------------------------
local function getTooltip(tt)
	local l,c,_,x = nil,nil,function(...) local l,c = tt:AddLine() for i,v in ipairs({...}) do tt:SetCell(l,i,v) end end,function(str,align) local l,c = tt:AddLine() tt:SetCell(l,1,str,nil,align,2) end

	l,c = tt:AddLine()
	tt:SetCell(l,1,C("dkyellow",L[name]),tt:GetHeaderFont(),nil,2)

	tt:AddSeparator(5,0,0,0,0)
	x(C("ltblue",L["Mail box status"]))
	tt:AddSeparator(1)
	local newMails = {GetLatestThreeSenders()}
	if #newMails>0 then
		x(C("ltyellow",L["Mail from"]..":"))
		for i,v in ipairs(newMails) do
			_(i..". "..v)
		end
	else
		x(L["No Mail"])
	end

	if Broker_EverythingDB[name].showDaysLeft then
		local hasDaysLeft = {}
		for i,v in pairs(mailDB) do
			for I,V in pairs(v) do
				if V.count>0 and V.daysLeft~=false then
					if (not hasDaysLeft[i]) then hasDaysLeft[i] = {} end
					hasDaysLeft[i][I] = V
				end
			end
		end

		tt:AddSeparator(5,0,0,0,0)
		x(C("ltblue",L["Mail returns to sender in"]))
		tt:AddSeparator(1)
		for i,v in pairs(hasDaysLeft) do
			x(C("ltyellow",i),"CENTER")
			for I,V in pairs(v) do
				if V.count>0 then
					_(C(V.class or "white",I),V.daysLeft.." "..L["Days"])
				end
			end
		end
	end
end


------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(obj)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
	if Broker_EverythingDB[name].hideMinimapMail then
		MiniMapMailFrame:Hide()
		MiniMapMailFrame.Show = dummyFunc
	end

	if (not mailDB[ns.realm]) then mailDB[ns.realm] = {} end
	if (not mailDB[ns.realm][ns.player.name]) then mailDB[ns.realm][ns.player.name] = {count=0,daysLeft=false,class=ns.player.class} end
end

ns.modules[name].onevent = function(self,event,msg)
	local dataobj = self.obj  or ns.LDB:GetDataObjectByName(ldbName)
	local newMail = false
	
	if event == "UPDATE_PENDING_MAIL" or event == "PLAYER_ENTERING_WORLD" or event =="PLAYER_LOGIN" then
--		local sender1, sender2, sender3 = GetLatestThreeSenders()
		if GetLatestThreeSenders()~=nil then
			newMail = true --HasNewMail() are really unsave to inform the player has mail in the mail box.
		end
		
		if unreadMail ~= newMail then
			if newMail and Broker_EverythingDB[name].playsound == true then
				PlaySoundFile("Interface\\Addons\\"..addon.."\\media\\mailalert.mp3", "Master")
			end
			unreadMail = newMail
		end
	
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:UnregisterEvent("PLAYER_LOGIN")
	end
	
	if event == "MAIL_INBOX_UPDATE" or event == "MAIL_SHOW" or event == "MAIL_CLOSED" then
		local num,_daysLeft = GetInboxNumItems(),99
		for i = 1, num do
			local _, _, _, _, _, _, daysLeft, _, wasRead, wasReturned, _, _, _ = GetInboxHeaderInfo(i);
			if wasReturned==nil and daysLeft < _daysLeft then
				_daysLeft = daysLeft
			end
			if( not wasRead ) then
				newMail = true;
				break;
			end
		end
		mailDB[ns.realm][ns.player.name] = {count=num,daysLeft=num>0 and _daysLeft~=99 and floor(_daysLeft) or false,class=ns.player.class}
	end

	local icon, text = I(name), L["No Mail"]
	if newMail then
		icon, text = I(name.."_new"), C("green",L["New Mail"])
	end
	dataobj.iconCoords = icon.coords or {0,1,0,1}
	dataobj.icon = icon.iconfile
	dataobj.text = text
end

--[[ ns.modules[name].onupdate = function(self) end ]]

--[[ ns.modules[name].optionspanel = function(panel) end ]]

--[[ ns.modules[name].onmousewheel = function(self,direction) end ]]

ns.modules[name].ontooltip = function(tooltip)
	local sender1, sender2, sender3 = GetLatestThreeSenders()
	ns.tooltipScaling(tooltip)
	tooltip:AddLine(L[name])

	tooltip:AddLine(" ")
	if not sender1 then
		tooltip:AddLine(L["No Mail"])
		return
	end

	tooltip:AddLine(L["Mail from"])
	tooltip:AddLine(ns.scm(sender1))
	if sender2 then tooltip:AddLine(ns.scm(sender2)) end
	if sender3 then tooltip:AddLine(ns.scm(sender3)) end

end


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
--[[
ns.modules[name].onenter = function(self)
	tt = ns.LQT:Acquire(name.."TT", 2, "LEFT", "RIGHT")
	getTooltip(tt)
	ns.createTooltip(self,tt)
end

ns.modules[name].onleave = function(self)
	if tt then
		if MouseIsOver(tt) then
			ns.hideTooltip(tt)
			tt:SetScript('OnLeave', ns.hideTooltip)
		else
			ns.hideTooltip(tt)
		end
	end
end
]]
--[[ ns.modules[name].onclick = function(self,button) end ]]

--[[ ns.modules[name].ondblclick = function(self,button) end ]]


-- MinimapMailFrameUpdate


--_G['MiniMapMailFrame']:HookScript("OnLoad",function(self) if Broker_EverythingDB[name].hideMinimapMail then self:UnregisterEvent("UPDATE_PENDING_MAIL") end end)
--_G['MiniMapMailFrame']:HookScript("OnEvent",function(self) if Broker_EverythingDB[name].hideMinimapMail then self:Hide() end end)


--[[

 -- DBR = Days Before Return
Broker_EverythingDB[name].showRealmCharsMailCount
Broker_EverythingDB[name].critDBR -- crit warn display in broker. in tooltip red
Broker_EverythingDB[name].warnDBR -- display in tooltip orange.

realm wide mail count with lowest days before return to sender
warn on critical "days before return" (choosable)

or realm independent? maybe... 
Broker_EverythingDB[name].showAllRealms :) let the use choose.

]]