
----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I
calendarDB = {}
twinkDB = {}


-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = (Broker_EverythingDB.usePrefix and "BE.." or "").."Time manager"
local ldbName = name
local dayStart,dayStop,dayN = time({year=0,month=0,day=0,hour=0,min=0,sec=0}),time({year=0,month=0,day=0,hour=23,min=59,sec=59}),86400
local GetGameTime,CalendarGetMonth = GetGameTime,CalendarGetMonth
local CalendarGetNumDayEvents, CalendarGetDayEvent = CalendarGetNumDayEvents, CalendarGetDayEvent
local scanTooltip = CreateFrame("GameTooltip",addon.."_"..name.."_ScanTooltip",UIParent,"GameTooltipTemplate")
scanTooltip:SetScale(0.0001)
scanTooltip:Hide()
local player, tt, tt2, counter
local numDungeons, numRaids, numChallenges, numUnknown
local instances, bosses = {},{}
local timer,counter = {},{}
local eTypes = {
	[CALENDAR_EVENTTYPE_RAID]    = L["Raid"],
	[CALENDAR_EVENTTYPE_DUNGEON] = L["Dungeon"],
	[CALENDAR_EVENTTYPE_PVP]     = L["PvP"],
	[CALENDAR_EVENTTYPE_MEETING] = L["Meeting"],
	[CALENDAR_EVENTTYPE_OTHER]   = L["Other"]
}
local iTypes = {
	[CALENDAR_INVITESTATUS_INVITED]      = L["Invited"],
	[CALENDAR_INVITESTATUS_ACCEPTED]     = L["Accepted"],
	[CALENDAR_INVITESTATUS_DECLINED]     = L["Declined"],
	[CALENDAR_INVITESTATUS_CONFIRMED]    = L["Confirmed"],
	[CALENDAR_INVITESTATUS_OUT]          = L["Out"],
	[CALENDAR_INVITESTATUS_STANDBY]      = L["Standby"],
	[CALENDAR_INVITESTATUS_SIGNEDUP]     = L["Signedup"],
	[CALENDAR_INVITESTATUS_NOT_SIGNEDUP] = L["Not signedup"],
	[CALENDAR_INVITESTATUS_TENTATIVE]    = L["Tentative"]
}
local custom_cTypes = {
	WORLDBOSS_KILL    = true,
	WORLDBOSS_BONUS   = true,
	RAIDBOSS_KILL     = true,
	RAIDBOSS_BONUS    = true
}


---------------------------------------
-- module variables for registration --
---------------------------------------
I[name] = {iconfile=[[interface\icons\inv_misc_pocketwatch_01]],coords={0.05,0.95,0.05,0.95}}

ns.modules[name] = {
	desc = L["-"],
	events = {
		--"CALENDAR_ACTION_PENDING",
		--"CALENDAR_CLOSE_EVENT",
		"CALENDAR_EVENT_ALARM",
		"CALENDAR_NEW_EVENT",
		--"CALENDAR_OPEN_EVENT",
		"CALENDAR_UPDATE_EVENT",
		"CALENDAR_UPDATE_EVENT_LIST",
		"CALENDAR_UPDATE_GUILD_EVENTS",
		"CALENDAR_UPDATE_INVITE_LIST",
		"CALENDAR_UPDATE_PENDING_INVITES",
		"PLAYER_ENTERING_WORLD"
	},
	updateinterval = 10,
	config_defaults = {
		format24 = true,
		showSeconds = false
	},
	config_allowed = {},
	config = {
		height = 62,
		elements = {
			{
				type = "check",
				name = "format24",
				label = L["24 hours mode"], -- TIMEMANAGER_24HOURMODE
				desc = L["Switch between time format 24 hours and 12 hours with AM/PM"],
			},
			{
				type = "check",
				name = "showSeconds",
				label = L["Show seconds"],
				desc = L["Display the time with seconds in broker button and tooltip"],
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x = 200,
					y = 0
				},
			},
		}
	}
}


--------------------------
-- some local functions --
--------------------------
local function formatTime(h,m,s)
	local a,h = "AM",tonumber(h)
	if (not Broker_EverythingDB[name].format24) then
		if h >= 12 then h, a = h - 12, "PM" end
		if h == 0 then h = 12 end
		if s~=nil then
			return ("%02d:%02d:%02s %s"):format(h, m, s, a)
		else
			return ("%02d:%02d %s"):format(h, m, a)
		end
	end
	if s~=nil then
		return ("%02d:%02d:%02s"):format(h,m,s)
	else
		return ("%02d:%02d"):format(h,m)
	end
end


local function updateInstances()
	numDungeons, numRaids,numChallenges,numUnknown = 0,0,0,0
	for i=1, GetNumSavedInstances() do
		local instanceName, instanceID, instanceReset, instanceDifficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(i)
		if instanceReset>0 then
			scanTooltip:Show()
			scanTooltip:SetOwner(UIParent,"LEFT",0,0)
			scanTooltip:SetInstanceLockEncountersComplete(i)
			local reg,data,line = {scanTooltip:GetRegions()},{},1
			local n,nc
			for k,v in pairs(reg) do
				if v~=nil and v:GetObjectType()=="FontString" and v:GetText()~=nil then
					if line>1 then
						if line/2==floor(line/2) then
							n = v:GetText()
						else
							tinsert(data,{n,({v:GetTextColor()})[0]~=0})
						end
					end
					line = line + 1
				end
			end
			scanTooltip:ClearLines()
			scanTooltip:Hide()
			if instanceDifficulty<=2 then
				numDungeons = numDungeons+1
			elseif instanceDifficulty==8 then
				numChallenges = numChallenges+1
			elseif instanceDifficulty<=9 then
				numRaids = numRaids+1
			else
				numUnknown = numUnknown+1
				ns.print(name,"unknown",numUnknown)
			end
			if instances[instanceDifficulty]==nil then instances[instanceDifficulty]={} end
			instances[instanceDifficulty][instanceID] = {
				instanceName		= instanceName,
				instanceReset		= instanceReset,
				extended			= extended,
				maxPlayers			= maxPlayers,
				difficultyName		= difficultyName,
				numEncounters		= numEncounters~=0 and numEncounters or "a",
				encounterProgress	= numEncounters~=0 and encounterProgress or "n",
				encounters			= data
			}
		end
	end
end

local function updateBosses()
	bosses = {}
	for i=1, GetNumSavedWorldBosses() do
		local n,id,r = GetSavedWorldBossInfo(i)
		local d = {name=n,id=id,reset=r}
		if type(r)=="number" and r>0 then
			tinsert(bosses,d)
		end
	end
end

local function getCount(field)
	updateInstances()
	updateBosses()
	if     field=="ev" then -- events
		
	elseif field=="ge" then -- guild events
		
	elseif field=="ga" then -- guild announcements
		
	elseif field=="wb" then -- world boss
		return #bosses
	elseif field=="ra" then -- raid (10/25/40)
		return numRaids
	elseif field=="rb" then -- raid (browser)
	elseif field=="rf" then -- raid (flexibel)
	elseif field=="du" then -- dungeon
		return numDungeons
	elseif field=="cm" then -- challenge mode
		return  numChallenges
	elseif field=="unknown" then
		return numUnknown
	end

	return "n/a"
end

local function scanCalenderDB() -- scan calendar entries from other chars collected by scanCalendar previously.
	
end

local function scanCalendar() -- scan calendar of current character
	local weekday, _month, day, year = CalendarGetDate()
	local t,h = {},{}

	-- catch all events from blizzards calendar
	for mOffset=0, 6 do
		local month, year, numdays, firstday = CalendarGetMonth(mOffset)
		for dayI = mOffset==0 and day or 1, numdays do
			for eventI=1, CalendarGetNumDayEvents(mOffset,dayI) do
				local title, hour, minute, calendarType, sequenceType, eventType, _, _, inviteStatus, invitedBy, _, inviteType, _, _, _ = CalendarGetDayEvent(mOffset, dayI, eventI)
				if calendarType~="HOLIDAY" and calendarType~="RAID_LOCKOUT" then
					local timeStamp = time({year=year, month=month, day=dayI, hour=hour, min=minute, sec=0})
					local data = {
						title     = title,
						cType     = calendarType,
						eType     = eventType,
						iStatus   = inviteStatus,
						iBy       = invitedBy,
						iType     = inviteType,
						timeStamp = timeStamp
					}
					tinsert(t,data)

					if timer[timeStamp]==nil then timer[timeStamp] = {} end
					tinsert(timer[timeStamp],player)

					local YMD = ("%d-%d-%d"):format(year,month,dayI)
					if counter[player]==nil then counter[player] = {[YMD]=0} end
					if counter[player][YMD]==nil then counter[player][YMD]=0 end
					counter[player][YMD] = counter[player][YMD] + 1
				end
			end
		end
	end

	-- copy custom cType events from previous table
	if calendarDB["charCalendar"][player]~=nil then
		for i,v in ipairs(calendarDB["charCalendar"][player]) do
			if custom_cTypes[v.cType]~=nil and v.timeStamp > GetTime() then
				tinsert(t,v)
			end
		end
	end
	calendarDB["charCalendar"][player] = t
end

local function tt2_instances(tt2,header,selection)
	local c,_ = 0,nil
	--[[
	tt2:AddHeader(C("dkyellow",header))
	tt2:AddSeparator(4,0,0,0,0)
	]]
	tt2:AddLine(C("ltblue",L["Name"]),C("ltblue",L["Type"]),C("ltblue",L["Bosses"]),C("ltblue",L["Reset in"]))
	tt2:AddSeparator()
	for _,I in ipairs(selection) do
		if type(instances[I])=="table" then
			for i,v in pairs(instances[I]) do
				if c>0 then tt2:AddSeparator() end
				tt2:AddLine(C("ltyellow",v.instanceName),v.difficultyName,("%s/%s"):format(v.encounterProgress,v.numEncounters),SecondsToTime(v.instanceReset))
				--[[
				if #v.encounters>0 then
					local x,y = {},0
					for _,d in ipairs(v.encounters) do
						y = y + strlen(d[1])
						if y<=60 then
							tinsert(x,C(d[2] and "red" or "green",d[1]))
						else
							y = 0
							tinsert(x,"|n"..C(d[2]and "red" or "green",d[1]))
						end
					end
					l,_ = tt2:AddLine()
					tt2:SetCell(l,1,table.concat(x,", "),nil,nil,3)
				end
				]]
				c = c+1
			end
		end
	end
end

local function tt2_calender(tt2,header,selection)
end

local function tt2_bosses(tt2,header)
	local c,_ = 0,nil
	tt2:AddHeader(C("dkyellow",header))
	tt2:AddSeparator(4,0,0,0,0)

	tt2:AddLine(C("ltblue",L["Name"]),"","",C("ltblue",L["Reset in"]))
	tt2:AddSeparator()
	for i,v in ipairs(bosses) do
		if c>0 then tt2:AddSeparator() end
		tt2:AddLine(v.name,"","",SecondsToTime(v.reset))
		c = c+1
	end
end

local function makeTooltip2(self, object)
	local l,c,_
	tt2 = ns.LQT:Acquire(name.."TT2", 4, "LEFT", "LEFT", "CENTER", "RIGHT")
	ns.createTooltip(self, tt2)
	tt2:ClearAllPoints()
	tt2:SetPoint("TOP",self,"TOP",0,0)
	tt2:Clear()

	local c = 0
	if     object=="ev" then -- events
		tt2:AddHeader(C("ltblue",L["Events"]))
		tt2:AddSeparator()

	elseif object=="ge" then -- guild events
		tt2:AddHeader(C("ltblue",L["Guild events"]))
		tt2:AddSeparator()

	elseif object=="ga" then -- guild announcements
		tt2:AddHeader(C("ltblue",L["Guild announcements"]))
		tt2:AddSeparator()

	elseif object=="wb" then -- world boss
		tt2_bosses(tt2,L["World bosses"])

	elseif object=="ra" then -- raid (10/25/40)
		tt2_instances(tt2,L["Raids"],{3,4,5,6,7,9})

	elseif object=="rb" then -- raid (browser)
		tt2:AddHeader(L["Raids (LFR)"])
		tt2:AddSeparator()

	elseif object=="rf" then -- raid (flexibel)
		tt2:AddHeader(L["Raids (Flex)"])
		tt2:AddSeparator()

	elseif object=="du" then -- dungeon
		tt2_instances(tt2,L["Dungeons"],{1,2})

	elseif object=="cm" then -- challenge mode
		tt2_instances(tt2,L["Challenge mode"],{8})

	end


	local tL,tR,tT,tB = ns.getBorderPositions(tt)
	local uW = UIParent:GetWidth()
	if tR<(uW/2) then
		tt2:SetPoint("RIGHT",tt,"LEFT",-2,0)
	else
		tt2:SetPoint("LEFT",tt,"RIGHT",2,0)
	end

end

local function leave_tt2(self)
	if tt2 then 
		if MouseIsOver(tt2) then
			ns.hideTooltip(tt2)
			tt2:SetScript('OnLeave', ns.hideTooltip)
		else
			ns.hideTooltip(tt2)
		end
	end
end

local function makeTooltip()
	if not (tt~=nil and tt.key~=nil and tt.key==name.."TT") then return end
	local l,c,h,m,s
	local clocks = {}
	h,m,s = ns.GetLocalTime()	clocks["local"]	= formatTime(h,m,s)
	h,m,s = ns.GetUTCTime()		clocks.utc		= formatTime(h,m,s)
	h,m,s = ns.GetGameTime()	clocks.server	= formatTime(h,m,s)

	tt:Clear()

	tt:AddHeader(C("dkyellow",L[name]))

	local ttlines = {
		{sep={3,0,0,0,0}},
		{line={C("ltblue",L["Time"])}},
		{sep={1}},
		{lines={C("ltyellow",L["Server:"]),	clocks.server}},
		{lines={C("ltyellow",L["Locale:"]),	clocks["local"]}},
		{lines={C("ltyellow",L["UTC:"]),	clocks.utc}},

		{sep={3,0,0,0,0}},
		{lines={C("ltblue",L["Today"]),C("ltblue",date("%Y-%m-%d"))}},
		{sep={1}},
		{lines={C("ltyellow",L["Events"]),              getCount("ev",0)}, tt2={nil,"ev",0}},
		{lines={C("ltyellow",L["Guild events"]),        getCount("ge",0)}, tt2={nil,"ge",0}},
		{lines={C("ltyellow",L["Guild announcements"]), getCount("ga",0)}, tt2={nil,"ga",0}},

		{sep={3,0,0,0,0}},
		{lines={C("ltblue",L["Future"])}},
		{sep={1}},
		{lines={C("ltyellow",L["Events"]),              getCount("ev",2)}, tt2={nil,"ev",2}},
		{lines={C("ltyellow",L["Guild events"]),        getCount("ge",2)}, tt2={nil,"ge",2}},
		{lines={C("ltyellow",L["Guild announcements"]), getCount("ga",2)}, tt2={nil,"ga",2}},

		{sep={3,0,0,0,0}},
		{lines={C("ltblue",L["ID's"]),C(ns.player.class,ns.player.name)}},
		{sep={1}},
		{lines={C("ltyellow",L["Worldbosses"]),         getCount("wb")}, tt2={nil,"wb"}},
		{lines={C("ltyellow",L["Raids (10/25)"]),       getCount("ra")}, tt2={nil,"ra"}},
		--{lines={C("ltyellow",L["Raids (Flex)"]),        getCount("fl")}, tt2={nil,"rf"}},
		--{lines={C("ltyellow",L["Raids (LFR)"]),         getCount("rb")}, tt2={nil,"rb"}},
		{lines={C("ltyellow",L["Dungeons"]),            getCount("du")}, tt2={nil,"du"}},
		--{lines={C("ltyellow",L["Challenge mode"]),      getCount("du")}, tt2={nil,"cm"}},
	}

	for i,v in ipairs(ttlines) do
		if v.sep~=nil then
			tt:AddSeparator(unpack(v.sep))
		elseif v.line~=nil then
			tt:AddLine(unpack(v.line))
		elseif v.lines~=nil then
			local l,c = tt:AddLine()
			for i,v in pairs(v.lines) do
				tt:SetCell(l,i,v,header==true and tt:GetHeaderFont() or nil)
			end
			if v.tt2 then
				tt:SetLineScript(l,"OnEnter",function(self) v.tt2[1]=self; makeTooltip2(unpack(v.tt2)); end)
				tt:SetLineScript(l,"OnLeave",leave_tt2)
			end
		end
	end

end

------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(obj)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
	if obj~=nil then return end
	player = GetUnitName("player", true)
end

ns.modules[name].onevent = function(self,...)
	local event = ...
	if event == "PLAYER_ENTERING_WORLD" then
		for i,v in pairs({charCalendar={[player]={}},raidids={},worldbossids={},flexbossids={},lfrbossids={},dungeonids={},lastchanged={}})
		do if calendarDB[i]==nil then calendarDB[i] = v end end
		if calendarDB["charCalendar"][player]==nil then calendarDB["charCalendar"][player]={} end
		scanCalenderDB()
		GameTimeFrame:Click()
		GameTimeFrame:Click()
	else
		scanCalendar()
	end

	-- collect events
	-- CalendarGetNumPendingInvites()
	-- local numEvents = CalendarGetNumDayEvents(monthOffset, day)
	-- http://wowpedia.org/API_CalendarGetNumDayEvents
		-- today
		-- tomorrow
			-- search long as 4 entries or 10 days

end

ns.modules[name].onupdate = function(self)
	if tt~=nil and tt.key==name.."TT" then
		makeTooltip(tt)
	end
end

--[[ ns.modules[name].optionspanel = function(panel) end ]]

--[[ ns.modules[name].onmousewheel = function(self,direction) end ]]

--[[ ns.modules[name].ontooltip = function(tooltip) end ]]


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
ns.modules[name].onenter = function(self)
	tt = ns.LQT:Acquire(name.."TT", 2, "LEFT", "RIGHT")
	makeTooltip(tt)
	ns.createTooltip(self, tt)
end

ns.modules[name].onleave = function(self)
	if tt then 
		if MouseIsOver(tt) then
			tt:SetScript('OnLeave', function(self) ns.hideTooltip(self,name) end)
		else
			ns.hideTooltip(tt,name)
		end
	end
end

--[[ ns.modules[name].onclick = function(self,button) end ]]

--[[ ns.modules[name].ondblclick = function(self,button) end ]]


--[=[
	

notes:
	time manager
		sep
		clock + date
			sep
			ServerTime: | <number>
			LocalTime:  | <number>
		calendar
			(realm/char independent calendar)
		worldbossloots
		raidid's (subTT: bosskills per raid?)
		char level playtime? ^^
]=]
