
----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.color, ns.L, ns.I


-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name = "Volume" -- L["Volume"]
local ldbName = name
local tt = nil
local ttColumns = 2
local icon = "Interface\\AddOns\\"..addon.."\\media\\volume_"
local volume = {}
local vol = {
	{inset=0,locale="MASTER_VOLUME",			toggle="Sound_EnableAllSound",								percent="Sound_MasterVolume"},
	{inset=1,locale="ENABLE_SOUNDFX",			toggle="Sound_EnableSFX",					depend={1},		percent="Sound_SFXVolume"},
	{inset=2,locale="ENABLE_ERROR_SPEECH",		toggle="Sound_EnableErrorSpeech",			depend={1,2}	},
	{inset=2,locale="ENABLE_EMOTE_SOUNDS",		toggle="Sound_EnableEmoteSounds",			depend={1,2}	},
	{inset=2,locale="ENABLE_PET_SOUNDS",		toggle="Sound_EnablePetSounds",				depend={1,2}	},
	{inset=1,locale="MUSIC_VOLUME",				toggle="Sound_EnableMusic",					depend={1},		percent="Sound_MusicVolume"},
	{inset=2,locale="ENABLE_MUSIC_LOOPING",		toggle="Sound_ZoneMusicNoDelay",			depend={1,6}	},
	{inset=2,locale="ENABLE_PET_BATTLE_MUSIC",	toggle="Sound_EnablePetBattleMusic",		depend={1,6}	},
	{inset=1,locale="ENABLE_AMBIENCE",			toggle="Sound_EnableAmbience",				depend={1},		percent="Sound_AmbienceVolume"},
	{inset=1,locale="ENABLE_BGSOUND",			toggle="Sound_EnableSoundWhenGameIsInBG",	depend={1}		},
	--{inset=1,locale="ENABLE_SOUND_AT_CHARACTER",toggle="Sound_ListenerAtCharacter",			depend={1}		},
	{inset=1,locale="ENABLE_REVERB",			toggle="Sound_EnableReverb",				depend={1}		},
	{inset=1,locale="ENABLE_SOFTWARE_HRTF",		toggle="Sound_EnableSoftwareHRTF",			depend={1}		},
	{inset=1,locale="ENABLE_DSP_EFFECTS",		toggle="Sound_EnableDSPEffects",			depend={1}		}
}


-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
I[name..'_0']    = {iconfile=icon.."0"}
I[name..'_33']   = {iconfile=icon.."33"}
I[name..'_66']   = {iconfile=icon.."66"}
I[name..'_100']  = {iconfile=icon.."100"}


---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name] = {
	desc = L["Change Volumes and toogle some audio options."],
	icon_suffix = "_100",
	events = {
		"ADDON_LOADED"
	},
	updateinterval = 5,
	config_defaults = {
		useWheel = false,
		steps = 10
	},
	config_allowed = {
	},
	config = {
		height = 62,
		elements = {
			{
				type = "check",
				name = "useWheel",
				label = L["Use MouseWheel"],
				desc = L["Use the MouseWheel to change the volume"]
			},
			{
				type = "slider",
				name = "steps",
				label = L["Steps"],
				desc = L["Change the stepping of volume changes."],
				minText = "1",
				maxText = "100",
				minValue = 1,
				maxValue = 100,
				default = 10,
				points = {
					edgeSelf = "TOPLEFT",
					edgeSibling = "TOPLEFT",
					x = 200,
					y = 0
				}
			}
		}
	}
}


--------------------------
-- some local functions --
--------------------------
local function updateBrokerButton(self)
	volume.master = tonumber(GetCVar("Sound_MasterVolume"))
	local obj = self.obj or ns.LDB:GetDataObjectByName(ldbName)
	local suffix = "100"
	if volume.master < .1 then
		suffix = "0"
	elseif volume.master < .3 then
		suffix = "33"
	elseif volume.master < .6 then
		suffix = "66"
	end
	local icon = I(name.."_"..(suffix or "100"))
	obj.iconCoords = icon.coords or {0,1,0,1}
	obj.icon = icon.iconfile
	obj.text = ceil(volume.master*100).."%"
end

local function volTooltip(tt)
	local l,c
	tt:Clear()
	tt:AddHeader(C("dkyellow",L[name]))
	tt:AddSeparator()

	local function percent(self,cvar,now,direction)
		if (direction==-1 and now==0) or (direction==1 and now==1) then return end
		local new = now + ((direction * Broker_EverythingDB[name].steps) / 100)
		new = (new>1 and 1) or (new<0 and 0) or new
		ns.SetCVar(cvar,new,cvar)
		volTooltip(tt)
		updateBrokerButton(self)
	end

	for i,v in ipairs(vol) do
		l,c = tt:AddLine()

		v.now = tonumber(GetCVar(v.toggle)) vol[i].now=v.now
		v.inv = v.now==1 and 0 or 1

		local color,disabled
		if v.depend~=nil and ( (v.depend[1]~=nil and vol[v.depend[1]].now==0) or (v.depend[2]~=nil and vol[v.depend[2]].now==0) ) then
			color = v.now==1 and "gray" or "dkgray"
			disabled = color
		end

		if color==nil then
			color = v.now==1 and "green" or "red"
			disabled = v.now==1 and "white" or "gray"
		end

		tt:SetCell(l,1,strrep(" ",3 * v.inset)..C(color,_G[v.locale]))
		tt:SetLineScript(l,"OnMouseUp",function(self, button) ns.SetCVar(v.toggle,tostring(v.inv),v.toggle) volTooltip(tt) end)

		if v.percent~=nil then
			v.pnow = tonumber(GetCVar(v.percent))

			tt.lines[l]:EnableMouseWheel(1)
			tt.lines[l]:SetScript("OnMouseWheel",function(self,direction)
				percent(self,v.percent,v.pnow,direction)
			end)

			tt:SetCell(l,ttColumns,C(disabled,ceil(v.pnow*100).."%"))
			tt:SetCellScript(l,ttColumns,"OnMouseUp",function(self,button) end)
			tt.lines[l].cells[ttColumns]:SetScript("OnMouseUp",function(self,button)
				local direction = button=="RightButton" and -1 or 1
				percent(self,v.percent,v.pnow,direction)
			end)
		else
			tt:SetCell(l,ttColumns,"           ")
		end
	end

	if Broker_EverythingDB.showHints then
		tt:AddSeparator(5,0,0,0,0)
		tt:AddLine(C("ltblue",L["Click"]).." || "..C("green",L["On/Off"]))
		tt:AddLine(C("ltblue",L["Mousewheel"]).." || "..C("green",L["Louder"].."/"..L["Quieter"]))
	end

end

------------------------------------
-- module (BE internal) functions --
------------------------------------
ns.modules[name].init = function(self)
	ldbName = (Broker_EverythingDB.usePrefix and "BE.." or "")..name
	if self then
		updateBrokerButton(self)
	end
end

--[[ ns.modules[name].onevent = function(self,event,msg) end ]]

ns.modules[name].onupdate = function(self)
	updateBrokerButton(self)
end

--[[ ns.modules[name].optionspanel = function(panel) end ]]

ns.modules[name].onmousewheel = function(self,direction)
	if not Broker_EverythingDB[name].useWheel then return end
	if (direction==-1 and volume.master == 0) or (direction==1 and volume.master == 1) then return end

	volume.master = volume.master + (direction * Broker_EverythingDB[name].steps / 100)
	if volume.master > 1 then
		volume.master = 1
	elseif volume.master < 0 then
		volume.master = 0
	end
	local cvar = "Sound_MasterVolume"
	ns.SetCVar(cvar,volume.master,cvar)
	--BlizzardOptionsPanel_SetCVarSafe("Sound_MasterVolume",volume.master)
	updateBrokerButton(self)
end


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
ns.modules[name].onenter = function(self)
	tt = ns.LQT:Acquire(name.."TT", 2, "LEFT", "RIGHT")
	tt:Clear()
	--ns.modules[name].ontooltip(tt)
	volTooltip(tt)
	ns.createTooltip(self,tt)
	ns.RegisterMouseWheel(self,ns.modules[name].onmousewheel)
end

ns.modules[name].onleave = function(self)
	if tt then 
		if MouseIsOver(tt) then
			tt:SetScript('OnLeave', function(self) ns.hideTooltip(self,name) end)
		else
			ns.hideTooltip(tt,name)
		end
	end
end

ns.modules[name].onclick = function(self,button)
	if button == "LeftButton" then
		if volume.master == 1 then return end
		volume.master = volume.master + (Broker_EverythingDB[name].steps / 100)
		if volume.master > 1 then volume.master = 1 elseif volume.master < 0 then volume.master = 0 end
		BlizzardOptionsPanel_SetCVarSafe("Sound_MasterVolume",volume.master)
		updateBrokerButton(self)
	elseif button == "RightButton" then
		if volume.master == 0 then return end
		volume.master = volume.master - (Broker_EverythingDB[name].steps / 100)
		if volume.master > 1 then volume.master = 1 elseif volume.master < 0 then volume.master = 0 end
		BlizzardOptionsPanel_SetCVarSafe("Sound_MasterVolume",volume.master)
		updateBrokerButton(self)
	end
end

--[[ ns.modules[name].ondblclick = function(self,button) end ]]





--[[

MasterVolume = 

self:SetOrientation('VERTICAL');
self:SetMinMaxValues(-1, 0);
self:SetValue((0-(MasterVolume/100)));
self:Show();

--DEFAULT_CHAT_FRAME:AddMessage("MasterVolume: "..MasterVolume);







self.value = abs(value);

if (self.value > .66) then
self.icon:SetTexCoord(.75, 1, 0, .25);
elseif (self.value > .33) then
self.icon:SetTexCoord(.50, .75, 0, .25);
elseif (self.value > 0) then
self.icon:SetTexCoord(.25, .50, 0, .25);
else
self.icon:SetTexCoord(0, .25, 0, .25);
end

self.label:SetText(tostring(ceil(self.value * 100)).."%");

ns.SetCVar("Sound_MasterVolume", self.value);



]]