
-- ====================================== --
-- Shared Functions for Broker_Everything --
-- ====================================== --
local addon, ns = ...
local upper,format,type = upper,format,type
local GetPlayerMapPosition,GetRealZoneText,GetSubZoneText = GetPlayerMapPosition,GetRealZoneText,GetSubZoneText
local GetZonePVPInfo,GetBindLocation = GetZonePVPInfo,GetBindLocation
local L = ns.L
local _
local build = tonumber(gsub(({GetBuildInfo()})[1],"[|.]","")..({GetBuildInfo()})[2])

ns.LDB = LibStub("LibDataBroker-1.1")
ns.LQT = LibStub("LibQTip-1.0")
ns.LDBI = LibStub("LibDBIcon-1.0")
ns.LSO = LibStub("LibSimpleOptions-1.0-be_mod")
ns.LSM = LibStub("LibSharedMedia-3.0")


-- ---------------------------------- --
-- realm dependent data               --
-- ~Hizuro                            --
-- ---------------------------------- --
ns.realm = GetRealmName()


-- ----------------------------------- --
-- player dependent data               --
-- ~Hizuro                             --
-- ----------------------------------- --
ns.player = {}
ns.player.name = UnitName("player")
_, ns.player.class = UnitClass("player")
ns.player.faction,ns.player.factionL  = UnitFactionGroup("player")
L[ns.player.faction] = ns.player.factionL
ns.player.female = UnitSex("player")==3
ns.player.classLocale = ns.player.female and _G.LOCALIZED_CLASS_NAMES_FEMALE[ns.player.class] or _G.LOCALIZED_CLASS_NAMES_MALE[ns.player.class]


-- ----------------------------------- --
-- SetCVar hook
-- Thanks at blizzard for blacklisting some cvars on combat...
-- ~Hizuro
-- ----------------------------------- --
do
	local blacklist = {alwaysShowActionBars = true, bloatnameplates = true, bloatTest = true, bloatthreat = true, consolidateBuffs = true, fullSizeFocusFrame = true, maxAlgoplates = true, nameplateMotion = true, nameplateOverlapH = true, nameplateOverlapV = true, nameplateShowEnemies = true, nameplateShowEnemyGuardians = true, nameplateShowEnemyPets = true, nameplateShowEnemyTotems = true, nameplateShowFriendlyGuardians = true, nameplateShowFriendlyPets = true, nameplateShowFriendlyTotems = true, nameplateShowFriends = true, repositionfrequency = true, showArenaEnemyFrames = true, showArenaEnemyPets = true, showPartyPets = true, showTargetOfTarget = true, targetOfTargetMode = true, uiScale = true, useCompactPartyFrames = true, useUiScale = true}
	ns.SetCVar = function(...)
		local cvar = ...
		if build>=54800000 and InCombatLockdown() and blacklist[cvar]==true then
			local msg
			if cvar=="uiScale" or cvar=="useUiScale" then
				msg = "Changing UI scaling while combat nether an good idea."
			else
				msg = "Sorry, CVar "..cvar.." are no longer changeable while combat. Thanks @ Blizzard."
			end
			print("|cffff8800"..addon..": "..msg.."|r")
		else
			SetCVar(...)
		end
	end
end


-- ----------------------------------- --
-- Helpful function for extra tooltips --
-- ----------------------------------- --
ns.onleaveTooltip = function(tooltip)
	if tooltip.tooltip then
		tooltip = tooltip.tooltip
	end
	if tooltip then
		if MouseIsOver(tooltip) then
			tooltip:SetScript("OnLeave", ns.hideTooltip)
		else
			ns.hideTooltip(tooltip)
		end
	end
end

local function GetTipAnchor(frame)
	local x, y = frame:GetCenter()
	if not x or not y then
		return "TOPLEFT", "BOTTOMLEFT"
	end
	local hhalf = (x > UIParent:GetWidth() * 2 / 3) and "RIGHT" or (x < UIParent:GetWidth() / 3) and "LEFT" or ""
	local vhalf = (y > UIParent:GetHeight() / 2) and "TOP" or "BOTTOM"
	local X = hhalf=="LEFT" and -3 or 3
	local Y = vhalf=="BOTTOM" and -3 or 3
	return vhalf .. hhalf, frame, (vhalf == "TOP" and "BOTTOM" or "TOP") .. hhalf, X,Y
end

local function SmartAnchorTo(self,frame)
	if not frame then
		error("Invalid frame provided.", 2)
	end
	self:ClearAllPoints()
	self:SetClampedToScreen(true)
	self:SetPoint(GetTipAnchor(frame))
end

ns.createTooltip = function(frame, tooltip, closer)
	if Broker_EverythingDB.tooltipScale == true then
		tooltip:SetScale(tonumber(GetCVar("uiScale")))
	end
	if frame then
		SmartAnchorTo(tooltip,frame)
	end
	tooltip:SetScript("OnLeave",ns.onleaveTooltip)
	tooltip:UpdateScrolling()
	tooltip:Show()
end

ns.tooltipScaling = function(tooltip)
	if Broker_EverythingDB.tooltipScale == true then
		tooltip:SetScale(tonumber(GetCVar("uiScale")))
	end
end

ns.hideTooltip = function(tooltip,ttName,ttForce,ttNoRelease)
	if tooltip~=nil then
		if ttForce~=true then
			if MouseIsOver(tooltip) then return end
			local f = GetMouseFocus()
			if f~=nil and (not f:IsForbidden()) and (not f:IsProtected() and InCombatLockdown()) and type(f.key)=="string" and type(ttName)=="string" and f.key:match(ttName) then return end
		end
		tooltip:SetScript('OnLeave', nil)
		if ttNoRelease==true then
			tooltip:ClearAllPoints();
			tooltip:Hide();
		else
			ns.LQT:Release(tooltip)
			ns.secureButton(false)
			tooltip = nil
		end
	end
end

ns.forceHideTooltip = function(tooltip)
	if tooltip~=nil then
		tooltip:SetScript('OnLeave',nil)
		ns.LQT:Release(tooltip)
		tooltip = nil
	end
end

ns.onleaveButton = function(self)
	if self.tooltip then
		if MouseIsOver(self.tooltip) then return end
		ns.hideTooltip(self.tooltip)
	end
end

ns.RegisterMouseWheel = function(self,func)
	self:EnableMouseWheel(1) 
	self:SetScript("OnMouseWheel", func)
end


-- -------------------------------------------------- --
-- a collection of color names and codes and a single --
-- function to return a text with a colorcode         --
-- ~Hizuro                                            --
-- -------------------------------------------------- --
--[[
	usage: ns.color2( <color> [, <string>] )
	<color> can be:
		colornames like red, green or so.
		colorcode like ffffff (google: html color hex)
		classnames like mage. localized or english. this function can both.

	if string nil this fucuntion
]]

do 
	local CbyN = { -- colors by names
		["yellow"]		= "ffffff00",
		["ltyellow"]	= "fffff569",
		["dkyellow"]	= "ffffcc00",
		["orange"]		= "ffff7d0a",
		["ltorange"]	= "ffff9d6a",
		["dkorange"]	= "ff905d0a",
		["dkred"]		= "ffc41f3b",
		["red"]			= "ffff0000",
		["ltred"]		= "ffff8080",
		["dkred"]		= "ff800000",
		["violet"]		= "fff000f0",
		["ltviolet"]	= "fff060f0",
		["dkviolet"]	= "ff800080",
		["blue"]		= "ff0000ff",
		["ltblue"]		= "ff69ccf0",
		["dkblue"]		= "ff000088",
		["cyan"]		= "ff00ffff",
		["ltcyan"]		= "ff80ffff",
		["dkcyan"]		= "ff008080",
		["green"]		= "ff00ff00",
		["ltgreen"]		= "ff80ff80",
		["dkgreen"]		= "ff00aa00",

		["black"]		= "ff000000",
		["dkgray"]		= "ff404040",
		["gray"]		= "ff808080",
		["ltgray"]		= "ffb0b0b0",
		["white"]		= "ffffffff",

		["gold"]		= "ffffd700",
		["silver"]		= "ffeeeeef",
		["copper"]		= "fff0a55f",

	}

	-- add class names to CbyN table plus localized names
	for n, c in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do CbyN[n:lower()] = c.colorStr end
	for k, v in pairs(_G.LOCALIZED_CLASS_NAMES_MALE) do CbyN[v:lower()] = CbyN[k:lower()] end
	for k, v in pairs(_G.LOCALIZED_CLASS_NAMES_FEMALE) do CbyN[v:lower()] = CbyN[k:lower()] end

	-- add item quality colors
	for i,v in pairs(ITEM_QUALITY_COLORS) do if i>=0 then CbyN["quality"..i] = ("ff%02x%02x%02x"):format(255*v.r, 255*v.g, 255*v.b) CbyN[_G["ITEM_QUALITY"..i.."_DESC"]:lower()] = CbyN["quality"..i] end end

	local function HexCode2ColorTable(colorStr)
		local codes = {string.sub(colorStr,3,4), string.sub(colorStr,5,6), string.sub(colorStr,7,8), string.sub(colorStr,1,2)}
		for i,v in pairs(codes) do
			v = string.format("%d","0x"..v)
			if v~=0 then codes[i] = ((100/255) * v) / 100 end
		end
		return codes
	end

	ns.rgb2Hex = function(t)
		return 
	end

	local function colorTable2HexCode(cTable)
		for i,v in pairs(cTable) do
			if type(v)~="number" then return "white" end
			v = string.format("%x",v * 100 / (100/255))
			if string.len(v)<2 then v = "0"..v end
			cTable[i] = v
		end
		return cTable[4]..cTable[1]..cTable[2]..cTable[3]
	end

	ns.color = function(color, str)
		if type(color)=="table" then color = colorTable2HexCode(color) end
		assert( type(color)=="string", "argument #1 must be a string. ("..type(color).." given)")
		color = color:lower()

		if color=="suffix" and Broker_EverythingDB.suffixColour then
			color = ns.player.class:lower()
		end

		if CbyN[color]~=nil then
			color = CbyN[color]
		elseif string.match(color, "^%x+$") then
			if color:len()==6 then color = "ff"..color end
		else
			color = "ffffffff"
		end

		if not str then
			return color
		elseif str=="colortable" then
			return HexCode2ColorTable(color)
		else
			return string.format("|c%s%s|r",color, str)
		end
	end

	local objs = {}
	ns.updateIconColor = function(name)
		local f = function(n)
			local obj = objs[n] or ns.LDB:GetDataObjectByName(n)
			objs[n] = obj
			if obj==nil then return false end
			obj.iconR,obj.iconG,obj.iconB,obj.iconA = unpack(Broker_EverythingDB.iconcolor or ns.C("white","colortable"))
			return true
		end
		if name==true then
			for i,v in pairs(ns.modules) do f(i) end
		elseif ns.modules[name]~=nil then
			f(name)
		end
	end
end


-- -------------------------- --
-- nice little print function --
-- ~Hizuro                    --
-- -------------------------- --
ns.print = function (...)
	local colors,t = {"red","green","ltblue","yellow","orange","violet"},{}
	for i,v in ipairs({addon..":",...}) do
		if type(v)=="string" and v:match("||c") then
			tinsert(t,v)
		else
			tinsert(t,ns.color(colors[i] or "white",v))
		end
	end
	print(unpack(t))
end
ns.Print = ns.print

ns.print_r = function(title,obj)
	assert(type(title)=="string","argument 1# must be a string ("..type(title).." given).")
	assert(type(obj)=="table","argument 2# must be a table ("..type(obj).." given)")
	for k,v in pairs(obj) do
		if type(v) ~= "string" and type(v)~="number" then v = "<"..type(v)..">" end
		ns.print(title,k,"=",v)
	end
end
ns.print_t = ns.print_r


-- -------------------------------------------------- --
-- Icon provider and framework to support             --
-- use of external iconset                            --
-- -------------------------------------------------- --
do
	local iconset = nil
	ns.I = setmetatable({},{
		__index = function(t,k)
			local v = {iconfile="interface\\icons\\inv_misc_questionmark",coords={0.05,0.95,0.05,0.95}}
			rawset(t, k, v)
			return v
		end,
		__call = function(t,a)
			if a==true then
				if Broker_EverythingDB.iconset~="NONE" then
					iconset = ns.LSM:Fetch((addon.."_Iconsets"):lower(),Broker_EverythingDB.iconset) or iconset
				end
				return
			end
			assert(type(a)=="string","argument #1 must be a string, got "..type(a))
			return (type(iconset)=="table" and iconset[a]) or t[a]
		end
	})
	ns.updateIcons = function()
		for i,v in pairs(ns.modules) do
			local obj = ns.LDB:GetDataObjectByName(i)
			if obj~=nil then 
				local d = ns.I(i .. (v.icon_suffix or ""))
				obj.iconCoords = d.coords or {0,1,0,1}
				obj.icon = d.iconfile
			end
		end
	end
end


-- -------------------------------------------------- --
-- Function to Sort a table by the keys               --
-- Sort function fom http://www.lua.org/pil/19.3.html --
-- -------------------------------------------------- --
ns.pairsByKeys = function(t, f)
	local a = {}
	for n in pairs(t) do
		table.insert(a, n)
	end
	table.sort(a, f)
	local i = 0      -- iterator variable
	local iter = function ()   -- iterator function
		i = i + 1
		if a[i] == nil then
			return nil
		else
			return a[i], t[a[i]]
		end
	end
	return iter
end

ns.reversePairsByKeys = function(t,f)
	local a = {}
	for n in ipairs(t) do
		table.insert(a,n)
	end
	table.sort(a, f)
	local i = #a
	local iter = function()
		i = i - 1
		if a[i] == nil then
			return nil
		else
			return a[i], t[a[i]]
		end
	end
	return iter
end


-- ---------------------------------------- --
-- Function to append an element to a table --
-- and optional limit his max entries       --
-- ~Hizuro                                  --
-- ---------------------------------------- --
ns.insertAppend = function(old_table,elem,max_entries)
	local new = {elem}
	for _, e in pairs(old_table) do
		new[#new] = e
		if max_entries ~= nil and #new == max_entries then return new end
	end
	return new
end


-- -------------------------------------------------------------- --
-- Function to split a string                                     --
-- http://stackoverflow.com/questions/1426954/split-string-in-lua --
-- Because mucking around with strings makes my head hurt.        --
-- -------------------------------------------------------------- --
ns.splitText = function(inputstr, sep)
	if sep == nil then
		sep = "%s"
	end
	local t={}
	local i = 1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
		t[i] = strtrim(str) .. sep
		i=i+1
	end
	return table.concat(t, "|n")
end

ns.splitTextToHalf = function(inputstr, sep)
	local t = {}
	local i = 1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
		t[i] =strtrim(str) .. " "
		i=i+1
	end
	local h,a,b = ceil(i/2),"",""
	for i, v in pairs(t) do
		if i < h then a = a .. v else b = b .. v end
	end
	return a.."|n"..b
end

ns.split = function(iStr,splitBy,opts)
	-- escapes sollten temporär ersetzt werden...
	if splitBy=="length" then
		assert(type(opts.length)=="number","opts.length must be a number, "..type(opts.length).."given.")
	elseif splitBy=="half" then
	elseif splitBy=="sepcount" then
		assert(type(opts.count)=="number","opts.count must be a number, "..type(opts.count).."given.")
	elseif splitBy=="sep" then
		
	end
end

ns.getBorderPositions = function(f)
	local us = UIParent:GetEffectiveScale()
	local uw,uh = UIParent:GetWidth(), UIParent:GetHeight()
	local fx,fy = f:GetCenter()
	local fw,fh = f:GetWidth()/2, f:GetHeight()/2
	-- LEFT, RIGHT, TOP, BOTTOM
	return fx-fw, uw-(fx+fw), uh-(fy+fh),fy-fh
end

ns.strLimit = function(str,limit)
	if strlen(str)>limit then str = strsub(str,1,limit).."..." end
	return str
end


-- -------------------------------------------------------------- --
-- module independent bag scan width interval                     --
-- ~Hizuro                                                        --
-- -------------------------------------------------------------- --
do
	ns.bagScan = {items={},last=time(),interval=10,active=false,resets={},updates={}}

	ns.bagScan.RegisterId = function(modName,itemId,foundFunc,resetFunc,updateFunc)
		assert(type(modName)=="string" and ns.modules[modName],"argument #1 (modName) must be a string, got "..type(modName))
		assert(type(itemId)=="number","argument #2 (itemId) must be a number, got "..type(itemId))
		assert(type(foundFunc)=="function","argument #3 (foundFunc) must be a function, got "..type(foundFunc))
		assert(type(resetFunc)=="function","argument #4 (resetFunc) must be a function, got "..type(resetFunc))
		assert(type(updateFunc)=="function","argument #5 (updateFunc) must be a function, got "..type(updateFunc))

		if ns.bagScan.items[itemId]==nil then
			ns.bagScan.items[itemId] = {}
		end
		ns.bagScan.items[itemId][modName] = foundFunc
		ns.bagScan.resets[modName] = resetFunc
		ns.bagScan.updates[modName] = updateFunc
		ns.bagScan.active = true
	end

	ns.bagScan.Update = function(now)
		-- prevent full usage of the function if no item/module registered.
		if ns.bagScan.active~=true then return end

		-- limit bagscan to a fix timeout. see ns.bagScan.interval
		if now~=true then
			if (time()-ns.bagScan.last)<ns.bagScan.interval then return end
			ns.bagScan.last = time()
		end

		-- reset tables in the registered modules
		for i,v in pairs(ns.bagScan.resets) do
			if type(v)=="function" then v() end
		end

		-- scan the bag and execute the foundFunc for all matching items
		for bag = 0, NUM_BAG_SLOTS do
			for slot = 1, GetContainerNumSlots(bag) do
				local item_id = GetContainerItemID(bag,slot)
				if ns.bagScan.items[item_id]~=nil then
					for i,v in pairs(ns.bagScan.items[item_id]) do v(item_id,bag,slot) end
				end
			end
		end

		-- trigger all updateFunc from all registered modules - mostly to update the broker button text
		for i,v in pairs(ns.bagScan.updates) do
			if type(v)=="function" then v() end
		end
	end
end


-- ----------------------------------------
-- secure button as transparent overlay
-- http://wowpedia.org/SecureActionButtonTemplate
-- be careful...
-- ~Hizuro
-- 
-- @param self UI_ELEMENT 
-- @param obj  TABLE
--		obj = {
--			{
--				typeName  STRING  | see "Modified attributes"
--				typeValue STRING  | see "Action types" "Type"-column
--				attrName  STRING  | see "Action types" "Used attributes"-column
--				attrValue ~mixed~ | see "Action types" "Behavior"-column. 
--				                  | Note: if typeValue is click then attrValue must
--										  be a ui element with :Click() function like
--										  buttons. thats a good way to open frames
--										  like spellbook without risk tainting it by
--										  an addon.
--			},
--			{ ... }
--		}
-- ----------------------------------------
do
	local sbf = nil -- change to array
	ns.secureButton = function(self,obj)
		if self==nil or  InCombatLockdown() then return end

		if sbf~=nil and self==false then
			sbf:SetParent(UIParent)
			sbf:ClearAllPoints()
			sbf:Hide()
			return 
		end

		if type(obj)~="table" then
			return
		end

		sbf = sbf or CreateFrame("Button",addon.."_SecureButton",UIParent,"SecureActionButtonTemplate")
		sbf:SetParent(self)
		sbf:SetPoint("CENTER")
		sbf:SetWidth(self:GetWidth())
		sbf:SetHeight(self:GetHeight())
		sbf:SetHighlightTexture([[interface\friendsframe\ui-friendsframe-highlightbar-blue]],true)

		for i,v in pairs(obj) do
			if type(v.typeName)=="string" and type(v.typeValue)=="string" then
				sbf:SetAttribute(v.typeName,v.typeValue)
			end
			if type(v.attrName)=="string" and v.attrValue~=nil then
				sbf:SetAttribute(v.attrName,v.attrValue)
			end
		end

		sbf:Show()
	end

	local sb = {}
	ns.secureButton2 = function(self,obj,name)
		if type(obj)~="table" then return end
		local sbf = nil

		if sb[name]==nil and (not InCombatLockdown()) then
			sb[name] = CreateFrame("Button",addon.."_SecureButton_"..name,self,"SecureActionButtonTemplate")

			--sb[name]:SetParent(self)
			if sb[name].SetOwner~=nil then
				sb[name]:SetOwner(self)
			end
			sb[name]:SetPoint("CENTER")
			sb[name]:SetWidth(self:GetWidth())
			sb[name]:SetHeight(self:GetHeight())

			sb[name]:SetHighlightTexture([[interface\friendsframe\ui-friendsframe-highlightbar-blue]],true)

			for i,v in pairs(obj) do
				if type(v.typeName)=="string" and type(v.typeValue)=="string" then
					sb[name]:SetAttribute(v.typeName,v.typeValue)
				end
				if type(v.attrName)=="string" and v.attrValue~=nil then
					sb[name]:SetAttribute(v.attrName,v.attrValue)
				end
			end

			sb[name]:Show()
		end
	end
end

-- --------------------- --
-- scanTooltip functions --
-- --------------------- --
do
	local scanTooltip = CreateFrame("GameTooltip",addon.."_ScanTooltip",UIParent,"GameTooltipTemplate")
	scanTooltip:SetScale(0.0001)
	scanTooltip:Hide()

	-- ------------------------------------------- --
	-- GetItemData                                 --
	-- a 2in1 function to fetch item informations  --
	-- for use in other addons                     --
	-- ~Hizuro                                     --
	-- ------------------------------------------- --
	function ns.GetItemData(id,bag,slot)
		assert(type(id)=="number","argument #1 (id) must be a number, got "..type(id))
		local data = {}
		data.itemName, data.itemLink, data.itemRarity, data.itemLevel, data.itemMinLevel, data.itemType, data.itemSubType, data.itemStackCount, data.itemEquipLoc, data.itemTexture, data.itemSellPrice = GetItemInfo(id)
		if bag~=nil and slot~=nil then
			assert(type(bag)=="number","argument #2 (bag) must be a number, got "..type(bag))
			assert(type(slot)=="number","argument #3 (slot) must be a number, got "..type(slot))
			data.startTime, data.duration, data.isEnabled = GetContainerItemCooldown(bag,slot)
			scanTooltip:Show()
			scanTooltip:SetOwner(UIParent,"LEFT",0,0)
			scanTooltip:SetBagItem(bag,slot)
			local reg = {scanTooltip:GetRegions()}
			local line = 1
			for k,v in pairs(reg) do
				if v~=nil and v:GetObjectType()=="FontString" and v:GetText()~=nil then
					data["tooltipLine"..line] = v:GetText()
					line = line + 1
				end
			end
			scanTooltip:ClearLines()
			scanTooltip:Hide()
		end
		return data
	end

	-- --------------------------------------- --
	-- GetRealFaction2PlayerStanding           --
	-- retrun standingID of a faction          --
	-- if faction unknown or argument nil then --
	-- returns this function the standingID 4  --
	-- --------------------------------------- --
	function ns.GetFaction2PlayerStanding(faction) -- FactionID or FactionName
		local collapsed, standing = {},4
		if faction~=nil then
			for i=GetNumFactions(), 1, -1 do -- 1. round: expand all collapsed headers
				local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, _, _, _ = GetFactionInfo(i)
				if isHeader and isCollapsed then
					collaped[name] = true
					ExpandFactionHeader(i)
				end
			end
			for i=1, GetNumFactions() do -- 2. round: search faction and note his standing
				local name, _, standingID, _, _, _, _, _, _, _, _, _, _, factionID, _, _ = GetFactionInfo(i)
				if faction==name or faction==factionID then
					standing = standingID
				end
			end
			for i=GetNumFactions(), 1, -1 do -- 3. round: collapsed all by this function expanded headers. 
				local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, _, _, _ = GetFactionInfo(i)
				if isHeader and collapsed[name] then
					CollapseFactionHeader(i)
				end
			end
		end
		return standing
	end

	-- ----------------
	-- UnitFaction
	-- ----------------
	function ns.UnitFaction(unit)
		scanTooltip:SetUnit(unit)
		scanTooltip:Show()
		local reg,_next,faction = {scanTooltip:GetRegions()},false,nil
		scanTooltip:Hide()
		for i,v in ipairs(reg) do
			if v:GetObjectType()=="FontString" then
				v = v:GetText() or ""
				if _next==false and v:match("^"..TOOLTIP_UNIT_LEVEL) then
					_next = true
				elseif _next==true then
					faction = v
					_next = nil
				end
			end
		end
		return faction, ns.GetFaction2PlayerStanding(faction)
	end
end

-- ----------------
-- tooltip graph (unstable)
-- ~Hizuro
-- ----------------
do
	local modData = {}
	ns.tooltipGraph = function(modName, maxValues, maxHeight, noScale)
		local s, barWidth, barSpace = UIParent:GetEffectiveScale(), 1, 1
		if noScale==true then s = 1 end

		local fN = ("%s_%s_ttGraph"):format(addon,modName)
		local f = f or _G[fN] or CreateFrame("frame",fN)
		if f.bars==nil then
			do
				f.bars = {}
				local n = f
				for i=1, maxValues do
					f.bars[i] = CreateFrame("frame",nil,f)
					f.bars[i]:SetWidth(barWidth * s)
					f.bars[i]:SetBackdrop({bgFile=[[Interface\Buttons\WHITE8X8]], Tile=false})
					if type(Broker_EverythingDB[modName].barColor)=="table" then
						f.bars[i]:SetBackdropColor(Broker_EverythingDB[modName].barColor)
					end
					f.bars[i]:SetPoint("BOTTOMLEFT", n, f==n and "BOTTOMLEFT" or "BOTTOMRIGHT", f==n and 0 or barSpace * s, 0)
					n=f.bars[i]
				end
			end
		end

		f:SetHeight(maxHeight)
		f:SetWidth(#modData[modName] * (barWidth+barSpace))

		local minV,maxV = 9999,0
		for i, v in ipairs(modData[modName]) do
			if v<minV then minV = v end --min value
			if v>maxV then maxV = v end --max value
		end

		local v
		for i=maxValues, 1, -1 do
			if modData[modName][i]==nil then
				v = 1
			else
				v = (modData[modName][i] - minV) / ((maxV - minV) / maxHeight)
			end
			f.bars[maxValues - i + 1]:SetHeight(ceil(v) * s)
		end

		return f
	end
	ns.tooltipGraphAddValue = function(modName,value,maxValues)
		local t = {}
		if modData[modName]==nil then modData[modName] = {} end
		table.insert(t,tonumber(value))
		for i,v in ipairs(modData[modName]) do
			if (#t < maxValues) then
				table.insert(t,v)
			end
		end
		modData[modName] = t
	end
end

-- ----------------------------------------------------- --
-- goldColor function to display amount of gold          --
-- in colored strings or with coin textures depending on --
-- a per module and a addon wide toggle.                 --
-- ~Hizuro                                               --
-- ----------------------------------------------------- --
function ns.GetCoinColorOrTextureString(modName,amount,sep)
	if Broker_EverythingDB[modName].goldColor==true or Broker_EverythingDB.goldColor==true then
		if (not sep) then sep = "." end
		local gold, silver, copper, t, i = floor(amount / 10000), mod(floor(amount / 100), 100), mod(floor(amount), 100), {}, 1
		if gold>0 then t[i]=ns.color("gold",gold) silver=("%02d"):format(silver) i=i+1 end
		if tonumber(silver)>0 or silver=="00" then t[i]=ns.color("silver",silver) copper=("%02d"):format(copper) i=i+1 end
		t[i] = ns.color("copper",copper)
		return table.concat(t,sep)
	else
		return GetCoinTextureString(amount)
	end
end


-- ----------------------------------------------------- --
-- screen capture mode - string replacement function     --
-- ~Hizuro                                               --
-- ----------------------------------------------------- --
ns.scm = function(str,all)
	if type(str)=="string" and strlen(str)>0 and Broker_EverythingDB.scm==true then
		if all then
			return strrep("*",(strlen(str)))
		else
			return strsub(str,1,1)..strrep("*",(strlen(str)-1))
		end
	else
		return str
	end
end


-- ----------------------------------------------------- --
-- Get[Game|Local|UTC|Country|Played]Time functions      --
-- ~Hizuro                                               --
-- ----------------------------------------------------- --
do
	local d = {frame=CreateFrame("frame"),sync=time(),m=false}
	d.frame.elapsed, d.frame.interval = 0, 0.5
	function ns.GetGameTime(syncIt)
		local h, m, s, n = GetGameTime()

		if s~=nil then -- surprise. blizzard provide server time with seconds? [maybe in future ^^]
			d.frame:SetScript("OnUpdate",nil)
			ns.GetGameTime = GetGameTime
			return h, m, s
		end

		if d.m==false then
			d.m=m
		end

		if syncIt==true and d.m~=m then
			d.frame.interval=1
			d.sync=time()
			d.m=m
		end

		return h, m, ("%02d"):format(time()-d.sync)
	end

	d.frame:SetScript("OnUpdate",function(self,elapsed) if self.elapsed>=self.interval then self.elapsed=0; ns.GetGameTime(true) end self.elapsed = self.elapsed + elapsed; end)
end

function ns.GetLocalTime()
	return date("%H"), date("%M"), date("%S")
end

function ns.GetUTCTime()
	return date("!%H"), date("!%M"), date("!%S")
end

--[[
function ns.GetCountryTime(country)
	local cdata
	if type(country)=="string" and ns.countriesBy.name[country]~=nil then
		cdata = ns.countries[ns.countriesBy.name[country] ]
	elseif type(country)=="number" and ns.countries[country]~=nil then
		cdata = ns.countries[country];
	end
	assert(type(cdata)=="table", "usage: ns.GetCountryTime(<country name or countryId>");
	
end
]]

do
	local d = CreateFrame("frame")
	d:SetScript("OnEvent",function(self,event,...) self.total, self.level = ...; self.stamp = time() end)
	d:RegisterEvent("TIME_PLAYED_MSG")
	function ns.GetPlayedTime()
		local session = time()-d.stamp
		return d.total + session, d.level + session, session -- rise up played time without use of RequestTimePlayed()
	end
end
